"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
/* istanbul ignore file */
const commander_1 = tslib_1.__importDefault(require("commander"));
const fs = tslib_1.__importStar(require("fs-extra"));
const path = tslib_1.__importStar(require("path"));
const glob_1 = tslib_1.__importDefault(require("glob"));
const readCode_1 = require("./thrift/readCode");
const print_1 = require("./thrift/print");
const combine_1 = tslib_1.__importDefault(require("./tools/combine"));
const updateNotify_1 = require("./tools/updateNotify");
const format_1 = require("./tools/format");
const packageJSON = JSON.parse(fs.readFileSync(path.resolve(__dirname, '../../package.json'), 'utf8'));
updateNotify_1.updateNotify(packageJSON);
commander_1.default
    .version(packageJSON.version)
    .usage('-p <thrift_dir> -o <typings_dir>')
    .option('-p, --project [dir]', 'thrift 根目录，默认为当前目录', process.cwd())
    .option('-an, --auto-namespace', '是否使用文件夹路径作为 namespace')
    .option('-s --strict', '如果字段没有指定 required 视为 optional')
    .option('--timestamp', '在头部加上生成时间')
    .option('-e --entry [filename]', '指定入口文件名', 'index.d.ts')
    .option('--use-tag <tagName>', '使用 tag 名称替换 field 名称')
    .option('--prettier', '输出时使用 prettier 格式化', false)
    .option('--rpc-namespace <rpc-namespace>', '指定一个独立的 namespace 存放所有 service', '')
    .option('-o, --out [out dir]', '输出 d.ts 文件根目录', path.resolve(process.cwd(), 'typings'));
commander_1.default.parse(process.argv);
const options = {
    root: path.resolve(process.cwd(), commander_1.default.project),
    tsRoot: path.resolve(process.cwd(), commander_1.default.out),
    entryName: path.resolve(process.cwd(), commander_1.default.out, commander_1.default.entry),
    autoNamespace: commander_1.default.autoNamespace,
    useStrictMode: commander_1.default.strict,
    useTimestamp: commander_1.default.timestamp,
    useTag: commander_1.default.useTag,
    usePrettier: commander_1.default.prettier,
    rpcNamespace: commander_1.default.rpcNamespace
};
fs.ensureDirSync(options.tsRoot);
fs.copyFileSync(path.join(__dirname, 'tools/tsHelper.d.ts'), path.join(options.tsRoot, 'tsHelper.d.ts'));
const thriftFiles = glob_1.default
    .sync('**/*.thrift', { cwd: options.root })
    .map(d => path.resolve(options.root, d));
const includeMap = {};
const tasks = thriftFiles.map(async (filename) => {
    let entity = null;
    try {
        entity = await readCode_1.readCode(filename, options, includeMap);
    }
    catch (e) {
        console.error(e);
        console.error(`read file fail.(${filename})`);
        return;
    }
    return entity;
});
const rTasks = [];
rTasks.push(Promise.all(tasks).then(entityList => {
    return Promise.all(entityList.map(async (entity) => {
        try {
            return print_1.print(entity, options, includeMap);
        }
        catch (e) {
            console.error(e);
            console.error(`write file fail.(${entity.fileName})`);
        }
    }));
}));
// summary RPC service
if (options.rpcNamespace) {
    rTasks.push(Promise.all(tasks).then(async (entityList) => {
        const allServices = [];
        const rtn = entityList.reduce((rtn, entity) => {
            if (!entity || !entity.services.length) {
                return rtn;
            }
            allServices.push(...entity.services);
            const r = print_1.printCollectionRpc(entity, includeMap);
            if (!r) {
                return rtn;
            }
            rtn += r;
            return rtn;
        }, '');
        const tarPath = path.join(options.tsRoot, options.rpcNamespace + '-rpc.d.ts');
        fs.writeFile(tarPath, format_1.prettier(`/// <reference path="./tsHelper.d.ts" />';

declare namespace ${options.rpcNamespace} {\n${rtn}
  export interface _Summary_ {
${allServices.map(d => `${d.name}: WrapperService<${d.name}>;`).join('\n  ')}
  }
}`), 'utf8', () => {
            console.log(`\u001b[32mFinished.\u001b[39m RPC d.ts files is \u001b[97m${tarPath}\u001b[39m.`);
        });
    }));
}
Promise.all(rTasks).then(async () => {
    combine_1.default(options);
    console.log(`\u001b[32mFinished.\u001b[39m Please check the d.ts files in \u001b[97m${options.tsRoot}\u001b[39m.`);
});
