"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * service ItemProviderService {
     RangeResponse    RangeItem            (1: RangeRequest req)  #总是按时间倒序
     // 根据时间来请求item信息
     ItemListResponse ItemListByCreateTime (1: RangeRequest req)  #总是按时间倒序
     // 根据ID来请求详情
     ItemListResponse ItemListByID         (1: ItemIDListRequest req)  #按照ID
   }
 *
 * @export
 * @param {string[]} codeLines
 */
function parseService(codeLines, _options) {
    // 兼容例如 `service ItemProviderSerivce {  RangeResponse    RangeItem (1: RangeRequest req)  #总是按时间倒序` 的格式
    formatServiceFirstLine(codeLines);
    const serviceName = isServiceBlockStart(codeLines[0]).mc;
    const rtn = {
        name: serviceName,
        interfaces: {}
    };
    let i = 1;
    for (; i < codeLines.length; i++) {
        const code = codeLines[i];
        parseServiceInterface(rtn.interfaces, code.trim());
    }
    return rtn;
}
exports.parseService = parseService;
function formatServiceFirstLine(lines) {
    const blockChars = lines
        .filter(d => !!d)
        .join('\n')
        .split('');
    const formattedChars = [];
    let chr;
    let commentStart = false;
    while ((chr = blockChars.shift())) {
        if (!commentStart &&
            (chr === '#' || (chr === '/' && blockChars[0] === '/'))) {
            commentStart = true;
        }
        if (commentStart === true && chr === '\n') {
            commentStart = false;
        }
        if (!commentStart && chr === '\n') {
            continue;
        }
        if (!commentStart) {
            formattedChars.push(chr);
        }
        if (!commentStart && chr === '{') {
            break;
        }
    }
    const fomattedLines = formattedChars
        .concat(blockChars)
        .join('')
        .split('\n');
    const newlines = fomattedLines.map(d => d.replace(/>\s*/g, ''));
    let firstLine = newlines[0];
    let cmt = '';
    firstLine = firstLine
        .replace(/(?:#|\/{2}).*$/, v => {
        cmt = v;
        return '';
    })
        .replace(/\{/, '{\n')
        .replace(/\}/, '\n}');
    const res = firstLine.split('\n');
    res[Math.min(res.length - 1, 1)] += cmt;
    return lines.splice(0, 1, ...res);
    // let char = '';
    // let i = 0;
    // const inserts = [];
    // while ((char = firstLine[i++])) {
    //   if ((char === '/' && firstLine[i] === '/') || char === '#') {
    //     // 如果发现是注释，再见
    //     break;
    //   }
    //   if (char === '{' && firstLine.substr(i).trim().length > 0) {
    //     inserts.push(lines[0] = firstLine.substr(0, i));
    //   }
    //   if (char === '}') {
    //     inserts.push('}');
    //   }
    // }
    // if (inserts.length > 0) {
    //   return lines.splice(1, 0, ...inserts);
    // } else {
    //   return lines;
    // }
}
exports.formatServiceFirstLine = formatServiceFirstLine;
const trapInterfaceRegExp = /^([^\s#\/]+)\s+([^\s\/]+)\s*\(([^\)]*)\)(?:\s+throws\(.+\))?[\s,;]?\s*(?:(?:#|\/\/)(.+))?$/;
function parseServiceInterface(interfaces, code) {
    const mc = trapInterfaceRegExp.exec(code);
    if (!mc) {
        return;
    }
    const inputParams = [];
    mc[3]
        .split(',')
        .map(d => d.trim())
        .filter(d => !!d)
        .forEach(d => {
        const _mc = d.match(/(\d+)\s*\:\s*(?:(?:([^\s]+)\s+)|([^\s]+>))(.+)$/);
        if (_mc) {
            inputParams.push({
                index: Number(_mc[1]),
                name: _mc[4],
                type: _mc[2]
            });
        }
    });
    interfaces[mc[2]] = {
        returnType: mc[1],
        inputParams,
        comment: (mc[4] || '').trim()
    };
}
exports.parseServiceInterface = parseServiceInterface;
const trapServiceNameRegExp = /^\s*service\s+([\w_][^\s]*)\s*\{/;
function isServiceBlockStart(code) {
    const mc = trapServiceNameRegExp.exec(code);
    if (mc) {
        return {
            hit: true,
            mc: mc[1]
        };
    }
    return {
        hit: false,
        mc: null
    };
}
exports.isServiceBlockStart = isServiceBlockStart;
